@@ -1,7 +1,7 @@
 /*
  * alpm.h
  *
- *  Copyright (c) 2002-2007 by Judd Vinet <jvinet@zeroflux.org>
+ *  Copyright (c) 2002-2008 by Judd Vinet <jvinet@zeroflux.org>
  *  Copyright (c) 2005 by Aurelien Foret <orelien@chez.com>
  *  Copyright (c) 2005 by Christian Hamar <krics@linuxforum.hu>
  *  Copyright (c) 2005, 2006 by Miklos Vajna <vmiklos@frugalware.org>
@@ -26,6 +26,7 @@
 extern "C" {
 #endif
 
+#include <sys/types.h> /* for off_t */
 #include <time.h> /* for time_t */
 #include <stdarg.h> /* for va_list */
 
@@ -45,14 +46,12 @@ typedef struct __pmdb_t pmdb_t;
 typedef struct __pmpkg_t pmpkg_t;
 typedef struct __pmdelta_t pmdelta_t;
 typedef struct __pmgrp_t pmgrp_t;
-typedef struct __pmserver_t pmserver_t;
 typedef struct __pmtrans_t pmtrans_t;
 typedef struct __pmsyncpkg_t pmsyncpkg_t;
 typedef struct __pmdepend_t pmdepend_t;
 typedef struct __pmdepmissing_t pmdepmissing_t;
 typedef struct __pmconflict_t pmconflict_t;
 typedef struct __pmfileconflict_t pmfileconflict_t;
-typedef struct __pmgraph_t pmgraph_t;
 
 /*
  * Library
@@ -81,21 +80,23 @@ int alpm_logaction(char *fmt, ...);
  * Downloading
  */
 
-typedef void (*alpm_cb_download)(const char *filename, int file_xfered,
-		int file_total, int list_xfered, int list_total);
+typedef void (*alpm_cb_download)(const char *filename,
+		off_t xfered, off_t total);
+typedef void (*alpm_cb_totaldl)(off_t total);
 
 /*
  * Options
  */
 
-#define PM_DLFNM_LEN 22
-
 alpm_cb_log alpm_option_get_logcb();
 void alpm_option_set_logcb(alpm_cb_log cb);
 
 alpm_cb_download alpm_option_get_dlcb();
 void alpm_option_set_dlcb(alpm_cb_download cb);
 
+alpm_cb_totaldl alpm_option_get_totaldlcb();
+void alpm_option_set_totaldlcb(alpm_cb_totaldl cb);
+
 const char *alpm_option_get_root();
 int alpm_option_set_root(const char *root);
 
@@ -170,7 +171,6 @@ int alpm_db_update(int level, pmdb_t *db);
 
 pmpkg_t *alpm_db_get_pkg(pmdb_t *db, const char *name);
 alpm_list_t *alpm_db_getpkgcache(pmdb_t *db);
-alpm_list_t *alpm_db_whatprovides(pmdb_t *db, const char *name);
 
 pmgrp_t *alpm_db_readgrp(pmdb_t *db, const char *name);
 alpm_list_t *alpm_db_getgrpcache(pmdb_t *db);
@@ -192,7 +192,7 @@ int alpm_pkg_load(const char *filename, unsigned short full, pmpkg_t **pkg);
 int alpm_pkg_free(pmpkg_t *pkg);
 int alpm_pkg_checkmd5sum(pmpkg_t *pkg);
 char *alpm_fetch_pkgurl(const char *url);
-int alpm_pkg_vercmp(const char *ver1, const char *ver2);
+int alpm_pkg_vercmp(const char *a, const char *b);
 alpm_list_t *alpm_pkg_compute_requiredby(pmpkg_t *pkg);
 
 const char *alpm_pkg_get_filename(pmpkg_t *pkg);
@@ -205,8 +205,8 @@ time_t alpm_pkg_get_installdate(pmpkg_t *pkg);
 const char *alpm_pkg_get_packager(pmpkg_t *pkg);
 const char *alpm_pkg_get_md5sum(pmpkg_t *pkg);
 const char *alpm_pkg_get_arch(pmpkg_t *pkg);
-unsigned long alpm_pkg_get_size(pmpkg_t *pkg);
-unsigned long alpm_pkg_get_isize(pmpkg_t *pkg);
+off_t alpm_pkg_get_size(pmpkg_t *pkg);
+off_t alpm_pkg_get_isize(pmpkg_t *pkg);
 pmpkgreason_t alpm_pkg_get_reason(pmpkg_t *pkg);
 alpm_list_t *alpm_pkg_get_licenses(pmpkg_t *pkg);
 alpm_list_t *alpm_pkg_get_groups(pmpkg_t *pkg);
@@ -225,38 +225,33 @@ size_t alpm_pkg_changelog_read(void *ptr, size_t size,
 int alpm_pkg_changelog_close(const pmpkg_t *pkg, void *fp);
 unsigned short alpm_pkg_has_scriptlet(pmpkg_t *pkg);
 
-unsigned long alpm_pkg_download_size(pmpkg_t *newpkg, pmdb_t *db_local);
+off_t alpm_pkg_download_size(pmpkg_t *newpkg);
 
 /*
  * Deltas
  */
 
 const char *alpm_delta_get_from(pmdelta_t *delta);
+const char *alpm_delta_get_from_md5sum(pmdelta_t *delta);
 const char *alpm_delta_get_to(pmdelta_t *delta);
-unsigned long alpm_delta_get_size(pmdelta_t *delta);
+const char *alpm_delta_get_to_md5sum(pmdelta_t *delta);
 const char *alpm_delta_get_filename(pmdelta_t *delta);
 const char *alpm_delta_get_md5sum(pmdelta_t *delta);
+off_t alpm_delta_get_size(pmdelta_t *delta);
 
 /*
  * Groups
  */
 const char *alpm_grp_get_name(const pmgrp_t *grp);
-const alpm_list_t *alpm_grp_get_pkgs(const pmgrp_t *grp);
+alpm_list_t *alpm_grp_get_pkgs(const pmgrp_t *grp);
 
 /*
  * Sync
  */
 
-/* Types */
-typedef enum _pmsynctype_t {
-	PM_SYNC_TYPE_REPLACE = 1,
-	PM_SYNC_TYPE_UPGRADE,
-	PM_SYNC_TYPE_DEPEND
-} pmsynctype_t;
-
-pmsynctype_t alpm_sync_get_type(const pmsyncpkg_t *sync);
 pmpkg_t *alpm_sync_get_pkg(const pmsyncpkg_t *sync);
-void *alpm_sync_get_data(const pmsyncpkg_t *sync);
+alpm_list_t *alpm_sync_get_removes(const pmsyncpkg_t *sync);
+pmpkg_t *alpm_sync_newversion(pmpkg_t *pkg, alpm_list_t *dbs_sync);
 int alpm_sync_sysupgrade(pmdb_t *db_local,
 		alpm_list_t *dbs_sync, alpm_list_t **syncpkgs);
 
@@ -266,10 +261,9 @@ int alpm_sync_sysupgrade(pmdb_t *db_local,
 
 /* Types */
 typedef enum _pmtranstype_t {
-	PM_TRANS_TYPE_ADD = 1,
+	PM_TRANS_TYPE_UPGRADE = 1,
 	PM_TRANS_TYPE_REMOVE,
 	PM_TRANS_TYPE_REMOVEUPGRADE,
-	PM_TRANS_TYPE_UPGRADE,
 	PM_TRANS_TYPE_SYNC
 } pmtranstype_t;
 
@@ -282,45 +276,103 @@ typedef enum _pmtransflag_t {
 	PM_TRANS_FLAG_CASCADE = 0x10,
 	PM_TRANS_FLAG_RECURSE = 0x20,
 	PM_TRANS_FLAG_DBONLY = 0x40,
-	PM_TRANS_FLAG_DEPENDSONLY = 0x80,
+	/* 0x80 flag can go here */
 	PM_TRANS_FLAG_ALLDEPS = 0x100,
 	PM_TRANS_FLAG_DOWNLOADONLY = 0x200,
 	PM_TRANS_FLAG_NOSCRIPTLET = 0x400,
 	PM_TRANS_FLAG_NOCONFLICTS = 0x800,
 	PM_TRANS_FLAG_PRINTURIS = 0x1000,
-	PM_TRANS_FLAG_NEEDED = 0x2000
+	PM_TRANS_FLAG_NEEDED = 0x2000,
+	PM_TRANS_FLAG_ALLEXPLICIT = 0x4000,
+	PM_TRANS_FLAG_UNNEEDED = 0x8000,
+	PM_TRANS_FLAG_RECURSEALL = 0x10000
 } pmtransflag_t;
 
-/* Transaction Events */
+/**
+ * @addtogroup alpm_trans
+ * @{
+ */
+/**
+ * @brief Transaction events.
+ * NULL parameters are passed to in all events unless specified otherwise.
+ */
 typedef enum _pmtransevt_t {
+	/** Dependencies will be computed for a package. */
 	PM_TRANS_EVT_CHECKDEPS_START = 1,
+	/** Dependencies were computed for a package. */
 	PM_TRANS_EVT_CHECKDEPS_DONE,
+	/** File conflicts will be computed for a package. */
 	PM_TRANS_EVT_FILECONFLICTS_START,
+	/** File conflicts were computed for a package. */
 	PM_TRANS_EVT_FILECONFLICTS_DONE,
+	/** Dependencies will be resolved for target package. */
 	PM_TRANS_EVT_RESOLVEDEPS_START,
+	/** Dependencies were resolved for target package. */
 	PM_TRANS_EVT_RESOLVEDEPS_DONE,
+	/** Inter-conflicts will be checked for target package. */
 	PM_TRANS_EVT_INTERCONFLICTS_START,
+	/** Inter-conflicts were checked for target package. */
 	PM_TRANS_EVT_INTERCONFLICTS_DONE,
+	/** Package will be installed.
+	 * A pointer to the target package is passed to the callback.
+	 */
 	PM_TRANS_EVT_ADD_START,
+	/** Package was installed.
+	 * A pointer to the new package is passed to the callback.
+	 */
 	PM_TRANS_EVT_ADD_DONE,
+	/** Package will be removed.
+	 * A pointer to the target package is passed to the callback.
+	 */
 	PM_TRANS_EVT_REMOVE_START,
+	/** Package was removed.
+	 * A pointer to the removed package is passed to the callback.
+	 */
 	PM_TRANS_EVT_REMOVE_DONE,
+	/** Package will be upgraded.
+	 * A pointer to the upgraded package is passed to the callback.
+	 */
 	PM_TRANS_EVT_UPGRADE_START,
+	/** Package was upgraded.
+	 * A pointer to the new package, and a pointer to the old package is passed
+	 * to the callback, respectively.
+	 */
 	PM_TRANS_EVT_UPGRADE_DONE,
-	PM_TRANS_EVT_EXTRACT_DONE,
+	/** Target package's integrity will be checked. */
 	PM_TRANS_EVT_INTEGRITY_START,
+	/** Target package's integrity was checked. */
 	PM_TRANS_EVT_INTEGRITY_DONE,
+	/** Target deltas's integrity will be checked. */
 	PM_TRANS_EVT_DELTA_INTEGRITY_START,
+	/** Target delta's integrity was checked. */
 	PM_TRANS_EVT_DELTA_INTEGRITY_DONE,
+	/** Deltas will be applied to packages. */
 	PM_TRANS_EVT_DELTA_PATCHES_START,
+	/** Deltas were applied to packages. */
 	PM_TRANS_EVT_DELTA_PATCHES_DONE,
+	/** Delta patch will be applied to target package.
+	 * The filename of the package and the filename of the patch is passed to the
+	 * callback.
+	 */
 	PM_TRANS_EVT_DELTA_PATCH_START,
+	/** Delta patch was applied to target package. */
 	PM_TRANS_EVT_DELTA_PATCH_DONE,
+	/** Delta patch failed to apply to target package. */
 	PM_TRANS_EVT_DELTA_PATCH_FAILED,
+	/** Scriptlet has printed information.
+	 * A line of text is passed to the callback.
+	 */
 	PM_TRANS_EVT_SCRIPTLET_INFO,
+	/** Print URI.
+	 * The database's URI and the package's filename are passed to the callback.
+	 */
 	PM_TRANS_EVT_PRINTURI,
+	/** Files will be downloaded from a repository.
+	 * The repository's tree name is passed to the callback.
+	 */
 	PM_TRANS_EVT_RETRIEVE_START,
 } pmtransevt_t;
+/*@}*/
 
 /* Transaction Conversations (ie, questions) */
 typedef enum _pmtransconv_t {
@@ -353,7 +405,6 @@ typedef void (*alpm_trans_cb_progress)(pmtransprog_t, const char *, int, int, in
 
 pmtranstype_t alpm_trans_get_type();
 unsigned int alpm_trans_get_flags();
-alpm_list_t * alpm_trans_get_targets();
 alpm_list_t * alpm_trans_get_pkgs();
 int alpm_trans_init(pmtranstype_t type, pmtransflag_t flags,
                     alpm_trans_cb_event cb_event, alpm_trans_cb_conv conv,
@@ -378,13 +429,16 @@ typedef enum _pmdepmod_t {
 	PM_DEP_MOD_LT
 } pmdepmod_t;
 
-pmdepend_t *alpm_splitdep(const char *depstring);
 int alpm_depcmp(pmpkg_t *pkg, pmdepend_t *dep);
 alpm_list_t *alpm_checkdeps(pmdb_t *db, int reversedeps,
 		alpm_list_t *remove, alpm_list_t *upgrade);
+alpm_list_t *alpm_deptest(pmdb_t *db, alpm_list_t *targets);
 
 const char *alpm_miss_get_target(const pmdepmissing_t *miss);
 pmdepend_t *alpm_miss_get_dep(pmdepmissing_t *miss);
+const char *alpm_miss_get_causingpkg(const pmdepmissing_t *miss);
+
+alpm_list_t *alpm_checkdbconflicts(pmdb_t *db_local);
 
 const char *alpm_conflict_get_package1(pmconflict_t *conflict);
 const char *alpm_conflict_get_package2(pmconflict_t *conflict);
@@ -439,12 +493,6 @@ enum _pmerrno_t {
 	PM_ERR_DB_REMOVE,
 	/* Servers */
 	PM_ERR_SERVER_BAD_URL,
-	/* Configuration */
-	PM_ERR_OPT_LOGFILE,
-	PM_ERR_OPT_DBPATH,
-	PM_ERR_OPT_LOCALDB,
-	PM_ERR_OPT_SYNCDB,
-	PM_ERR_OPT_USESYSLOG,
 	/* Transactions */
 	PM_ERR_TRANS_NOT_NULL,
 	PM_ERR_TRANS_NULL,
@@ -460,14 +508,12 @@ enum _pmerrno_t {
 	PM_ERR_PKG_INVALID,
 	PM_ERR_PKG_OPEN,
 	PM_ERR_PKG_LOAD,
-	PM_ERR_PKG_INSTALLED,
 	PM_ERR_PKG_CANT_FRESH,
 	PM_ERR_PKG_CANT_REMOVE,
 	PM_ERR_PKG_INVALID_NAME,
-	PM_ERR_PKG_CORRUPTED,
 	PM_ERR_PKG_REPO_NOT_FOUND,
 	/* Deltas */
-	PM_ERR_DLT_CORRUPTED,
+	PM_ERR_DLT_INVALID,
 	PM_ERR_DLT_PATCHFAILED,
 	/* Groups */
 	PM_ERR_GRP_NOT_FOUND,
@@ -478,14 +524,14 @@ enum _pmerrno_t {
 	/* Misc */
 	PM_ERR_USER_ABORT,
 	PM_ERR_INTERNAL_ERROR,
-	PM_ERR_LIBARCHIVE_ERROR,
 	PM_ERR_DB_SYNC,
 	PM_ERR_RETRIEVE,
 	PM_ERR_PKG_HOLD,
 	PM_ERR_INVALID_REGEX,
-  /* Downloading */
-	PM_ERR_CONNECT_FAILED,
-  PM_ERR_FORK_FAILED
+	/* External library errors */
+	PM_ERR_LIBARCHIVE,
+	PM_ERR_LIBDOWNLOAD,
+	PM_ERR_EXTERNAL_DOWNLOAD
 };
 
 extern enum _pmerrno_t pm_errno;
