@@ -31,7 +31,6 @@
 #include "log.h"
 #include "alpm.h"
 #include "util.h"
-#include "error.h"
 #include "package.h"
 #include "group.h"
 #include "db.h"
@@ -41,32 +40,21 @@
  */
 int _alpm_db_load_pkgcache(pmdb_t *db)
 {
-	pmpkg_t *info;
-	int count = 0;
-
 	ALPM_LOG_FUNC;
 
 	if(db == NULL) {
 		return(-1);
 	}
-
 	_alpm_db_free_pkgcache(db);
 
 	_alpm_log(PM_LOG_DEBUG, "loading package cache for repository '%s'\n",
-	          db->treename);
-
-	_alpm_db_rewind(db);
-	while((info = _alpm_db_scan(db, NULL)) != NULL) {
-		_alpm_log(PM_LOG_FUNCTION, "adding '%s' to package cache for db '%s'\n",
-							alpm_pkg_get_name(info), db->treename);
-		info->origin = PKG_FROM_CACHE;
-		info->origin_data.db = db;
-		/* add to the collection */
-		db->pkgcache = alpm_list_add(db->pkgcache, info);
-		count++;
+			db->treename);
+	if(_alpm_db_populate(db) == -1) {
+		_alpm_log(PM_LOG_DEBUG,
+				"failed to load package cache for repository '%s'\n", db->treename);
+		return(-1);
 	}
 
-	db->pkgcache = alpm_list_msort(db->pkgcache, count, _alpm_pkg_cmp);
 	return(0);
 }
 
@@ -81,10 +69,7 @@ void _alpm_db_free_pkgcache(pmdb_t *db)
 	_alpm_log(PM_LOG_DEBUG, "freeing package cache for repository '%s'\n",
 	                        db->treename);
 
-	alpm_list_t *tmp;
-	for(tmp = db->pkgcache; tmp; tmp = alpm_list_next(tmp)) {
-		_alpm_pkg_free(tmp->data);
-	}
+	alpm_list_free_inner(db->pkgcache, (alpm_list_fn_free)_alpm_pkg_free);
 	alpm_list_free(db->pkgcache);
 	db->pkgcache = NULL;
 
@@ -113,6 +98,7 @@ alpm_list_t *_alpm_db_get_pkgcache(pmdb_t *db)
 	return(db->pkgcache);
 }
 
+/* "duplicate" pkg with BASE info (to spare some memory) then add it to pkgcache */
 int _alpm_db_add_pkgincache(pmdb_t *db, pmpkg_t *pkg)
 {
 	pmpkg_t *newpkg;
@@ -123,10 +109,21 @@ int _alpm_db_add_pkgincache(pmdb_t *db, pmpkg_t *pkg)
 		return(-1);
 	}
 
-	newpkg = _alpm_pkg_dup(pkg);
+	newpkg = _alpm_pkg_new();
 	if(newpkg == NULL) {
 		return(-1);
 	}
+	newpkg->name = strdup(pkg->name);
+	newpkg->version = strdup(pkg->version);
+	if(newpkg->name == NULL || newpkg->version == NULL) {
+		pm_errno = PM_ERR_MEMORY;
+		_alpm_pkg_free(newpkg);
+		return(-1);
+	}
+	newpkg->origin = PKG_FROM_CACHE;
+	newpkg->origin_data.db = db;
+	newpkg->infolevel = INFRQ_BASE; 
+
 	_alpm_log(PM_LOG_DEBUG, "adding entry '%s' in '%s' cache\n",
 						alpm_pkg_get_name(newpkg), db->treename);
 	db->pkgcache = alpm_list_add_sorted(db->pkgcache, newpkg, _alpm_pkg_cmp);
@@ -181,7 +178,7 @@ pmpkg_t *_alpm_db_get_pkgfromcache(pmdb_t *db, const char *target)
 		return(NULL);
 	}
 
-	return(_alpm_pkg_find(target, pkgcache));
+	return(_alpm_pkg_find(pkgcache, target));
 }
 
 /* Returns a new group cache from db.
@@ -208,33 +205,29 @@ int _alpm_db_load_grpcache(pmdb_t *db)
 		pmpkg_t *pkg = lp->data;
 
 		for(i = alpm_pkg_get_groups(pkg); i; i = i->next) {
-			if(!alpm_list_find_str(db->grpcache, i->data)) {
-				pmgrp_t *grp = _alpm_grp_new();
+			const char *grpname = i->data;
+			alpm_list_t *j;
+			pmgrp_t *grp = NULL;
+			int found = 0;
 
-				strncpy(grp->name, i->data, GRP_NAME_LEN);
-				grp->name[GRP_NAME_LEN-1] = '\0';
-				grp->packages = alpm_list_add_sorted(grp->packages,
-																						 /* gross signature forces us to
-																							* discard const */
-																						 (void*)alpm_pkg_get_name(pkg),
-																						 _alpm_str_cmp);
-				db->grpcache = alpm_list_add_sorted(db->grpcache, grp, _alpm_grp_cmp);
-			} else {
-				alpm_list_t *j;
+			/* first look through the group cache for a group with this name */
+			for(j = db->grpcache; j; j = j->next) {
+				grp = j->data;
 
-				for(j = db->grpcache; j; j = j->next) {
-					pmgrp_t *grp = j->data;
-
-					if(strcmp(grp->name, i->data) == 0) {
-						const char *pkgname = alpm_pkg_get_name(pkg);
-						if(!alpm_list_find_str(grp->packages, pkgname)) {
-							grp->packages = alpm_list_add_sorted(grp->packages,
-							                                     (void*)pkgname,
-																									 _alpm_str_cmp);
-						}
-					}
+				if(strcmp(grp->name, grpname) == 0
+						&& !alpm_list_find_ptr(grp->packages, pkg)) {
+					grp->packages = alpm_list_add(grp->packages, pkg);
+					found = 1;
+					break;
 				}
 			}
+			if(found) {
+				continue;
+			}
+			/* we didn't find the group, so create a new one with this name */
+			grp = _alpm_grp_new(grpname);
+			grp->packages = alpm_list_add(grp->packages, pkg);
+			db->grpcache = alpm_list_add(db->grpcache, grp);
 		}
 	}
 
@@ -252,10 +245,6 @@ void _alpm_db_free_grpcache(pmdb_t *db)
 	}
 
 	for(lg = db->grpcache; lg; lg = lg->next) {
-		pmgrp_t *grp = lg->data;
-
-		alpm_list_free(grp->packages);
-		grp->packages = NULL;
 		_alpm_grp_free(lg->data);
 		lg->data = NULL;
 	}
