@@ -40,7 +40,7 @@ typedef enum _pmdbinfrq_t {
 /* Database */
 struct __pmdb_t {
 	char *path;
-	char treename[PATH_MAX];
+	char *treename;
 	void *handle;
 	alpm_list_t *pkgcache;
 	alpm_list_t *grpcache;
@@ -55,21 +55,13 @@ alpm_list_t *_alpm_db_search(pmdb_t *db, const alpm_list_t *needles);
 pmdb_t *_alpm_db_register_local(void);
 pmdb_t *_alpm_db_register_sync(const char *treename);
 
-/* Provision */
-int _alpm_prov_cmp(const void *provision, const void *needle);
-alpm_list_t *_alpm_db_whatprovides(pmdb_t *db, const char *package);
-
 /* be.c, backend specific calls */
-int _alpm_db_install(pmdb_t *db, const char *dbfile);
 int _alpm_db_open(pmdb_t *db);
 void _alpm_db_close(pmdb_t *db);
-void _alpm_db_rewind(pmdb_t *db);
-pmpkg_t *_alpm_db_scan(pmdb_t *db, const char *target);
+int _alpm_db_populate(pmdb_t *db);
 int _alpm_db_read(pmdb_t *db, pmpkg_t *info, pmdbinfrq_t inforeq);
 int _alpm_db_write(pmdb_t *db, pmpkg_t *info, pmdbinfrq_t inforeq);
 int _alpm_db_remove(pmdb_t *db, pmpkg_t *info);
-time_t _alpm_db_getlastupdate(const pmdb_t *db);
-int _alpm_db_setlastupdate(const pmdb_t *db, time_t time);
 
 #endif /* _ALPM_DB_H */
 
