@@ -43,114 +43,11 @@
 /* libalpm */
 #include "util.h"
 #include "log.h"
-#include "error.h"
 #include "package.h"
 #include "alpm.h"
 #include "alpm_list.h"
 #include "md5.h"
 
-#ifndef HAVE_STRVERSCMP
-/* GNU's strverscmp() function, taken from glibc 2.3.2 sources
- */
-
-/* Compare strings while treating digits characters numerically.
-   Copyright (C) 1997, 2002 Free Software Foundation, Inc.
-   Contributed by Jean-François Bignolles <bignolle@ecoledoc.ibp.fr>, 1997.
-
-   The GNU C Library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Lesser General Public
-   License as published by the Free Software Foundation; either
-   version 2.1 of the License, or (at your option) any later version.
-*/
-
-/* states: S_N: normal, S_I: comparing integral part, S_F: comparing
-           fractionnal parts, S_Z: idem but with leading Zeroes only */
-#define  S_N    0x0
-#define  S_I    0x4
-#define  S_F    0x8
-#define  S_Z    0xC
-
-/* result_type: CMP: return diff; LEN: compare using len_diff/diff */
-#define  CMP    2
-#define  LEN    3
-
-/* Compare S1 and S2 as strings holding indices/version numbers,
-   returning less than, equal to or greater than zero if S1 is less than,
-   equal to or greater than S2 (for more info, see the texinfo doc).
-*/
-
-int strverscmp (s1, s2)
-     const char *s1;
-     const char *s2;
-{
-  const unsigned char *p1 = (const unsigned char *) s1;
-  const unsigned char *p2 = (const unsigned char *) s2;
-  unsigned char c1, c2;
-  int state;
-  int diff;
-
-  /* Symbol(s)    0       [1-9]   others  (padding)
-     Transition   (10) 0  (01) d  (00) x  (11) -   */
-  static const unsigned int next_state[] =
-  {
-      /* state    x    d    0    - */
-      /* S_N */  S_N, S_I, S_Z, S_N,
-      /* S_I */  S_N, S_I, S_I, S_I,
-      /* S_F */  S_N, S_F, S_F, S_F,
-      /* S_Z */  S_N, S_F, S_Z, S_Z
-  };
-
-  static const int result_type[] =
-  {
-      /* state   x/x  x/d  x/0  x/-  d/x  d/d  d/0  d/-
-                 0/x  0/d  0/0  0/-  -/x  -/d  -/0  -/- */
-
-      /* S_N */  CMP, CMP, CMP, CMP, CMP, LEN, CMP, CMP,
-                 CMP, CMP, CMP, CMP, CMP, CMP, CMP, CMP,
-      /* S_I */  CMP, -1,  -1,  CMP, +1,  LEN, LEN, CMP,
-                 +1,  LEN, LEN, CMP, CMP, CMP, CMP, CMP,
-      /* S_F */  CMP, CMP, CMP, CMP, CMP, LEN, CMP, CMP,
-                 CMP, CMP, CMP, CMP, CMP, CMP, CMP, CMP,
-      /* S_Z */  CMP, +1,  +1,  CMP, -1,  CMP, CMP, CMP,
-                 -1,  CMP, CMP, CMP
-  };
-
-  if (p1 == p2)
-    return 0;
-
-  c1 = *p1++;
-  c2 = *p2++;
-  /* Hint: '0' is a digit too.  */
-  state = S_N | ((c1 == '0') + (isdigit (c1) != 0));
-
-  while ((diff = c1 - c2) == 0 && c1 != '\0')
-    {
-      state = next_state[state];
-      c1 = *p1++;
-      c2 = *p2++;
-      state |= (c1 == '0') + (isdigit (c1) != 0);
-    }
-
-  state = result_type[state << 2 | (((c2 == '0') + (isdigit (c2) != 0)))];
-
-  switch (state)
-  {
-    case CMP:
-      return diff;
-
-    case LEN:
-      while (isdigit (*p1++))
-	if (!isdigit (*p2++))
-	  return 1;
-
-      return isdigit (*p2) ? -1 : diff;
-
-    default:
-      return state;
-  }
-}
-#endif
-
 #ifndef HAVE_STRSEP
 /* This is a replacement for strsep which is not portable (missing on Solaris).
  * Copyright (c) 2001 by François Gouget <fgouget_at_codeweavers.com> */
@@ -186,57 +83,64 @@ int _alpm_makepath(const char *path)
 /* does the same thing as 'mkdir -p' */
 int _alpm_makepath_mode(const char *path, mode_t mode)
 {
-	char *orig, *str, *ptr;
-	char full[PATH_MAX] = "";
-	mode_t oldmask;
-
-	oldmask = umask(0000);
+	/* A bit of pointer hell here. Descriptions:
+	 * orig - a copy of path so we can safely butcher it with strsep
+	 * str - the current position in the path string (after the delimiter)
+	 * ptr - the original position of str after calling strsep
+	 * incr - incrementally generated path for use in stat/mkdir call
+	 */
+	char *orig, *str, *ptr, *incr;
+	mode_t oldmask = umask(0000);
+	int ret = 0;
 
 	orig = strdup(path);
+	incr = calloc(strlen(orig) + 1, sizeof(char));
 	str = orig;
 	while((ptr = strsep(&str, "/"))) {
 		if(strlen(ptr)) {
-			struct stat buf;
-
-			strcat(full, "/");
-			strcat(full, ptr);
-			if(stat(full, &buf)) {
-				if(mkdir(full, mode)) {
-					FREE(orig);
-					umask(oldmask);
-					_alpm_log(PM_LOG_ERROR, _("failed to make path '%s' : %s\n"),
-										path, strerror(errno));
-					return(1);
+			/* we have another path component- append the newest component to
+			 * existing string and create one more level of dir structure */
+			strcat(incr, "/");
+			strcat(incr, ptr);
+			if(access(incr, F_OK)) {
+				if(mkdir(incr, mode)) {
+					ret = 1;
+					break;
 				}
 			}
 		}
 	}
-	FREE(orig);
+	free(orig);
+	free(incr);
 	umask(oldmask);
-	return(0);
+	return(ret);
 }
 
+#define CPBUFSIZE 8 * 1024
+
 int _alpm_copyfile(const char *src, const char *dest)
 {
 	FILE *in, *out;
 	size_t len;
-	char buf[4097];
+	char *buf;
+	int ret = 0;
 
-	in = fopen(src, "r");
+	in = fopen(src, "rb");
 	if(in == NULL) {
 		return(1);
 	}
-	out = fopen(dest, "w");
+	out = fopen(dest, "wb");
 	if(out == NULL) {
 		fclose(in);
 		return(1);
 	}
 
+	CALLOC(buf, (size_t)CPBUFSIZE, (size_t)1, ret = 1; goto cleanup;);
+
 	/* do the actual file copy */
-	while((len = fread(buf, 1, 4096, in))) {
+	while((len = fread(buf, 1, CPBUFSIZE, in))) {
 		fwrite(buf, 1, len, out);
 	}
-	fclose(in);
 
 	/* chmod dest to permissions of src, as long as it is not a symlink */
 	struct stat statbuf;
@@ -246,12 +150,14 @@ int _alpm_copyfile(const char *src, const char *dest)
 		}
 	} else {
 		/* stat was unsuccessful */
-		fclose(out);
-		return(1);
+		ret = 1;
 	}
 
+cleanup:
+	fclose(in);
 	fclose(out);
-	return(0);
+	FREE(buf);
+	return(ret);
 }
 
 /* Trim whitespace and newlines from a string
@@ -371,6 +277,13 @@ int _alpm_lckrm()
 
 /* Compression functions */
 
+/**
+ * @brief Unpack a specific file or all files in an archive.
+ *
+ * @param archive  the archive to unpack
+ * @param prefix   where to extract the files
+ * @param fn       a file within the archive to unpack or NULL for all
+ */
 int _alpm_unpack(const char *archive, const char *prefix, const char *fn)
 {
 	int ret = 1;
@@ -382,7 +295,7 @@ int _alpm_unpack(const char *archive, const char *prefix, const char *fn)
 	ALPM_LOG_FUNC;
 
 	if((_archive = archive_read_new()) == NULL)
-		RET_ERR(PM_ERR_LIBARCHIVE_ERROR, -1);
+		RET_ERR(PM_ERR_LIBARCHIVE, -1);
 
 	archive_read_support_compression_all(_archive);
 	archive_read_support_format_all(_archive);
@@ -408,13 +321,17 @@ int _alpm_unpack(const char *archive, const char *prefix, const char *fn)
 			archive_entry_set_mode(entry, 0755);
 		}
 
+		/* If a specific file was requested skip entries that don't match. */
 		if (fn && strcmp(fn, entryname)) {
+			_alpm_log(PM_LOG_DEBUG, "skipping: %s\n", entryname);
 			if (archive_read_data_skip(_archive) != ARCHIVE_OK) {
 				ret = 1;
 				goto cleanup;
 			}
 			continue;
 		}
+
+		/* Extract the archive entry. */
 		ret = 0;
 		snprintf(expath, PATH_MAX, "%s/%s", prefix, entryname);
 		archive_entry_set_pathname(entry, expath);
@@ -513,12 +430,11 @@ int _alpm_logaction(unsigned short usesyslog, FILE *f, const char *fmt, va_list
 int _alpm_ldconfig(const char *root)
 {
 	char line[PATH_MAX];
-	struct stat buf;
 
 	snprintf(line, PATH_MAX, "%setc/ld.so.conf", root);
-	if(stat(line, &buf) == 0) {
+	if(access(line, F_OK) == 0) {
 		snprintf(line, PATH_MAX, "%ssbin/ldconfig", root);
-		if(stat(line, &buf) == 0) {
+		if(access(line, X_OK) == 0) {
 			char cmd[PATH_MAX];
 			snprintf(cmd, PATH_MAX, "%s -r %s", line, root);
 			system(cmd);
@@ -535,13 +451,12 @@ int _alpm_str_cmp(const void *s1, const void *s2)
 	return(strcmp(s1, s2));
 }
 
-/** Find a package file in an alpm cachedir.
- * @param filename name of package file to find
+/** Find a filename in a registered alpm cachedir.
+ * @param filename name of file to find
  * @return malloced path of file, NULL if not found
  */
 char *_alpm_filecache_find(const char* filename)
 {
-	struct stat buf;
 	char path[PATH_MAX];
 	char *retpath;
 	alpm_list_t *i;
@@ -550,8 +465,7 @@ char *_alpm_filecache_find(const char* filename)
 	for(i = alpm_option_get_cachedirs(); i; i = alpm_list_next(i)) {
 		snprintf(path, PATH_MAX, "%s%s", (char*)alpm_list_getdata(i),
 				filename);
-		if(stat(path, &buf) == 0) {
-			/* TODO maybe check to make sure it is readable? */
+		if(access(path, R_OK) == 0) {
 			retpath = strdup(path);
 			_alpm_log(PM_LOG_DEBUG, "found cached pkg: %s\n", retpath);
 			return(retpath);
@@ -640,13 +554,7 @@ char SYMEXPORT *alpm_get_md5sum(const char *filename)
 	ret = md5_file(filename, output);
 
 	if (ret > 0) {
-		if (ret == 1) {
-			_alpm_log(PM_LOG_ERROR, _("md5: %s can't be opened\n"), filename);
-		} else if (ret == 2) {
-			_alpm_log(PM_LOG_ERROR, _("md5: %s can't be read\n"), filename);
-		}
-
-		return(NULL);
+		RET_ERR(PM_ERR_NOT_A_FILE, NULL);
 	}
 
 	/* Convert the result to something readable */
@@ -660,4 +568,52 @@ char SYMEXPORT *alpm_get_md5sum(const char *filename)
 	return(md5sum);
 }
 
+int _alpm_test_md5sum(const char *filepath, const char *md5sum)
+{
+	char *md5sum2;
+	int ret;
+
+	md5sum2 = alpm_get_md5sum(filepath);
+
+	if(md5sum == NULL || md5sum2 == NULL) {
+		ret = -1;
+	} else if(strcmp(md5sum, md5sum2) != 0) {
+		ret = 1;
+	} else {
+		ret = 0;
+	}
+
+	FREE(md5sum2);
+	return(ret);
+}
+
+char *_alpm_archive_fgets(char *line, size_t size, struct archive *a)
+{
+	/* for now, just read one char at a time until we get to a
+	 * '\n' char. we can optimize this later with an internal
+	 * buffer. */
+	/* leave room for zero terminator */
+	char *last = line + size - 1;
+	char *i;
+
+	for(i = line; i < last; i++) {
+		int ret = archive_read_data(a, i, 1);
+		/* special check for first read- if null, return null,
+		 * this indicates EOF */
+		if(i == line && (ret <= 0 || *i == '\0')) {
+			return(NULL);
+		}
+		/* check if read value was null or newline */
+		if(ret <= 0 || *i == '\0' || *i == '\n') {
+			last = i + 1;
+			break;
+		}
+	}
+
+	/* always null terminate the buffer */
+	*last = '\0';
+
+	return(line);
+}
+
 /* vim: set ts=2 sw=2 noet: */
