@@ -3,7 +3,7 @@ self.description = "Remove a package in HoldPkg"
 p1 = pmpkg("dummy")
 self.addpkg2db("local", p1)
 
-self.option["holdpkg"] = ["dummy"]
+self.option["HoldPkg"] = ["dummy"]
 
 self.args = "-R %s" % p1.name
 
