@@ -15,6 +15,6 @@ self.addpkg2db("local", lp2)
 
 self.args = "-S %s" % " ".join([p.name for p in sp1, sp2])
 
-self.addrule("PACMAN_RETCODE=0")
+self.addrule("PACMAN_RETCODE=1")
 self.addrule("PKG_EXIST=pkg1")
-self.addrule("!PKG_EXIST=pkg2")
+self.addrule("PKG_EXIST=pkg2")
