@@ -19,6 +19,12 @@
 
 #include "config.h"
 
+/* special handling of package version for GIT */
+#if defined(GIT_VERSION)
+#undef PACKAGE_VERSION
+#define PACKAGE_VERSION GIT_VERSION
+#endif
+
 #include <stdlib.h> /* atoi */
 #include <stdio.h>
 #include <limits.h>
@@ -68,31 +74,27 @@ static void usage(int op, const char * const myname)
 		printf("%s:\n", str_opt);
 		printf("    %s {-h --help}\n", myname);
 		printf("    %s {-V --version}\n", myname);
-		printf("    %s {-A --add}     [%s] <%s>\n", myname, str_opt, str_file);
 		printf("    %s {-Q --query}   [%s] [%s]\n", myname, str_opt, str_pkg);
 		printf("    %s {-R --remove}  [%s] <%s>\n", myname, str_opt, str_pkg);
 		printf("    %s {-S --sync}    [%s] [%s]\n", myname, str_opt, str_pkg);
 		printf("    %s {-U --upgrade} [%s] <%s>\n", myname, str_opt, str_file);
 		printf(_("\nuse '%s --help' with other options for more syntax\n"), myname);
 	} else {
-		if(op == PM_OP_ADD) {
-			printf("%s:  %s {-A --add} [%s] <%s>\n", str_usg, myname, str_opt, str_file);
-			printf("%s:\n", str_opt);
-			printf(_("      --asdeps         install packages as non-explicitly installed\n"));
-			printf(_("  -d, --nodeps         skip dependency checks\n"));
-			printf(_("  -f, --force          force install, overwrite conflicting files\n"));
-		} else if(op == PM_OP_REMOVE) {
+		if(op == PM_OP_REMOVE) {
 			printf("%s:  %s {-R --remove} [%s] <%s>\n", str_usg, myname, str_opt, str_pkg);
 			printf("%s:\n", str_opt);
 			printf(_("  -c, --cascade        remove packages and all packages that depend on them\n"));
 			printf(_("  -d, --nodeps         skip dependency checks\n"));
 			printf(_("  -k, --dbonly         only remove database entry, do not remove files\n"));
 			printf(_("  -n, --nosave         remove configuration files as well\n"));
-			printf(_("  -s, --recursive      remove dependencies also (that won't break packages)\n"));
+			printf(_("  -s, --recursive      remove dependencies also (that won't break packages)\n"
+				 "                       (-ss includes explicitly installed dependencies too)\n"));
+			printf(_("  -u, --unneeded       remove unneeded packages (that won't break packages)\n"));
 		} else if(op == PM_OP_UPGRADE) {
 			printf("%s:  %s {-U --upgrade} [%s] <%s>\n", str_usg, myname, str_opt, str_file);
 			printf("%s:\n", str_opt);
 			printf(_("      --asdeps         install packages as non-explicitly installed\n"));
+			printf(_("      --asexplicit     install packages as explicitly installed\n"));
 			printf(_("  -d, --nodeps         skip dependency checks\n"));
 			printf(_("  -f, --force          force install, overwrite conflicting files\n"));
 		} else if(op == PM_OP_QUERY) {
@@ -115,9 +117,9 @@ static void usage(int op, const char * const myname)
 			printf("%s:  %s {-S --sync} [%s] [%s]\n", str_usg, myname, str_opt, str_pkg);
 			printf("%s:\n", str_opt);
 			printf(_("      --asdeps         install packages as non-explicitly installed\n"));
+			printf(_("      --asexplicit     install packages as explicitly installed\n"));
 			printf(_("  -c, --clean          remove old packages from cache directory (-cc for all)\n"));
 			printf(_("  -d, --nodeps         skip dependency checks\n"));
-			printf(_("  -e, --dependsonly    install dependencies only\n"));
 			printf(_("  -f, --force          force install, overwrite conflicting files\n"));
 			printf(_("  -g, --groups         view all members of a package group\n"));
 			printf(_("  -i, --info           view package information\n"));
@@ -151,11 +153,11 @@ static void version(void)
 {
 	printf("\n");
 	printf(" .--.                  Pacman v%s - libalpm v%s\n", PACKAGE_VERSION, alpm_version());
-	printf("/ _.-' .-.  .-.  .-.   Copyright (C) 2002-2008 Judd Vinet <jvinet@zeroflux.org>\n");
-	printf("\\  '-. '-'  '-'  '-'\n");
+	printf("/ _.-' .-.  .-.  .-.   Copyright (C) 2006-2008 Dan McGee <dan@archlinux.org>\n");
+	printf("\\  '-. '-'  '-'  '-'   Copyright (C) 2002-2006 Judd Vinet <jvinet@zeroflux.org>\n");
 	printf(" '--'\n");
 	printf(_("                       This program may be freely redistributed under\n"
-	         "                       the terms of the GNU General Public License\n"));
+	         "                       the terms of the GNU General Public License.\n"));
 	printf("\n");
 }
 
@@ -212,14 +214,15 @@ static void cleanup(int ret) {
  * in a consistant state.
  * @param signum the thrown signal
  */
-static void handler(int signum)
+static RETSIGTYPE handler(int signum)
 {
 	if(signum==SIGSEGV)
 	{
 		/* write a log message and write to stderr */
-		pm_printf(PM_LOG_ERROR, "segmentation fault\n");
-		pm_fprintf(stderr, PM_LOG_ERROR, "Internal pacman error: Segmentation fault.\n"
-		        "Please submit a full bug report with --debug if appropriate.\n");
+		pm_printf(PM_LOG_ERROR, _("segmentation fault\n"));
+		pm_fprintf(stderr, PM_LOG_ERROR,
+				_("Internal pacman error: Segmentation fault.\n"
+					"Please submit a full bug report with --debug if appropriate.\n"));
 		exit(signum);
 	} else if((signum == SIGINT)) {
 		if(alpm_trans_interrupt() == 0) {
@@ -305,7 +308,6 @@ static int parseargs(int argc, char *argv[])
 	int option_index = 0;
 	static struct option opts[] =
 	{
-		{"add",        no_argument,       0, 'A'},
 		{"query",      no_argument,       0, 'Q'},
 		{"remove",     no_argument,       0, 'R'},
 		{"sync",       no_argument,       0, 'S'},
@@ -318,7 +320,6 @@ static int parseargs(int argc, char *argv[])
 		{"clean",      no_argument,       0, 'c'},
 		{"nodeps",     no_argument,       0, 'd'},
 		{"deps",       no_argument,       0, 'd'},
-		{"dependsonly",no_argument,       0, 'e'},
 		{"explicit",   no_argument,       0, 'e'},
 		{"force",      no_argument,       0, 'f'},
 		{"groups",     no_argument,       0, 'g'},
@@ -338,6 +339,7 @@ static int parseargs(int argc, char *argv[])
 		{"unrequired", no_argument,       0, 't'},
 		{"upgrades",   no_argument,       0, 'u'},
 		{"sysupgrade", no_argument,       0, 'u'},
+		{"unneeded",   no_argument,       0, 'u'},
 		{"verbose",    no_argument,       0, 'v'},
 		{"downloadonly", no_argument,     0, 'w'},
 		{"refresh",    no_argument,       0, 'y'},
@@ -352,10 +354,11 @@ static int parseargs(int argc, char *argv[])
 		{"logfile",    required_argument, 0, 1009},
 		{"ignoregroup", required_argument, 0, 1010},
 		{"needed",     no_argument,       0, 1011},
+		{"asexplicit",     no_argument,   0, 1012},
 		{0, 0, 0, 0}
 	};
 
-	while((opt = getopt_long(argc, argv, "ARUFQSTr:b:vkhscVfmnoldepqituwygz", opts, &option_index))) {
+	while((opt = getopt_long(argc, argv, "RUFQSTr:b:vkhscVfmnoldepqituwygz", opts, &option_index))) {
 		alpm_list_t *list = NULL, *item = NULL; /* lists for splitting strings */
 
 		if(opt < 0) {
@@ -423,7 +426,9 @@ static int parseargs(int argc, char *argv[])
 				FREELIST(list);
 				break;
 			case 1011: config->flags |= PM_TRANS_FLAG_NEEDED; break;
-			case 'A': config->op = (config->op != PM_OP_MAIN ? 0 : PM_OP_ADD); break;
+			case 1012:
+				config->flags |= PM_TRANS_FLAG_ALLEXPLICIT;
+				break;
 			case 'Q': config->op = (config->op != PM_OP_MAIN ? 0 : PM_OP_QUERY); break;
 			case 'R': config->op = (config->op != PM_OP_MAIN ? 0 : PM_OP_REMOVE); break;
 			case 'S': config->op = (config->op != PM_OP_MAIN ? 0 : PM_OP_SYNC); break;
@@ -444,7 +449,6 @@ static int parseargs(int argc, char *argv[])
 				break;
 			case 'e':
 				config->op_q_explicit = 1;
-				config->flags |= PM_TRANS_FLAG_DEPENDSONLY;
 				break;
 			case 'f': config->flags |= PM_TRANS_FLAG_FORCE; break;
 			case 'g': (config->group)++; break;
@@ -468,7 +472,11 @@ static int parseargs(int argc, char *argv[])
 			case 's':
 				config->op_s_search = 1;
 				config->op_q_search = 1;
-				config->flags |= PM_TRANS_FLAG_RECURSE;
+				if(config->flags & PM_TRANS_FLAG_RECURSE) {
+					config->flags |= PM_TRANS_FLAG_RECURSEALL;
+				} else {
+					config->flags |= PM_TRANS_FLAG_RECURSE;
+				}
 				break;
 			case 't':
 				config->op_q_unrequired = 1;
@@ -476,6 +484,7 @@ static int parseargs(int argc, char *argv[])
 			case 'u':
 				config->op_s_upgrade = 1;
 				config->op_q_upgrade = 1;
+				config->flags |= PM_TRANS_FLAG_UNNEEDED;
 				break;
 			case 'v': (config->verbose)++; break;
 			case 'w':
@@ -512,6 +521,11 @@ static int parseargs(int argc, char *argv[])
 	return(0);
 }
 
+/* helper for being used with setrepeatingoption */
+static void option_add_syncfirst(const char *name) {
+	config->syncfirst = alpm_list_add(config->syncfirst, strdup(name));
+}
+
 /** Add repeating options such as NoExtract, NoUpgrade, etc to libalpm
  * settings. Refactored out of the parseconfig code since all of them did
  * the exact same thing and duplicated code.
@@ -599,7 +613,7 @@ static int _parseconfig(const char *file, const char *givensection,
 			}
 		} else {
 			/* directive */
-			char *key, *upperkey;
+			char *key;
 			/* strsep modifies the 'line' string: 'key \0 ptr' */
 			key = line;
 			ptr = line;
@@ -612,11 +626,7 @@ static int _parseconfig(const char *file, const char *givensection,
 						file, linenum);
 				return(1);
 			}
-			/* For each directive, compare to the uppercase and camelcase string.
-			 * This prevents issues with certain locales where characters don't
-			 * follow the toupper() rules we may expect, e.g. tr_TR where i != I.
-			 */
-			upperkey = strtoupper(strdup(key));
+			/* For each directive, compare to the camelcase string. */
 			if(section == NULL) {
 				pm_printf(PM_LOG_ERROR, _("config file %s, line %d: All directives must belong to a section.\n"),
 						file, linenum);
@@ -624,25 +634,22 @@ static int _parseconfig(const char *file, const char *givensection,
 			}
 			if(ptr == NULL && strcmp(section, "options") == 0) {
 				/* directives without settings, all in [options] */
-				if(strcmp(key, "NoPassiveFTP") == 0 || strcmp(upperkey, "NOPASSIVEFTP") == 0) {
+				if(strcmp(key, "NoPassiveFTP") == 0) {
 					alpm_option_set_nopassiveftp(1);
 					pm_printf(PM_LOG_DEBUG, "config: nopassiveftp\n");
-				} else if(strcmp(key, "UseSyslog") == 0 || strcmp(upperkey, "USESYSLOG") == 0) {
+				} else if(strcmp(key, "UseSyslog") == 0) {
 					alpm_option_set_usesyslog(1);
 					pm_printf(PM_LOG_DEBUG, "config: usesyslog\n");
-				} else if(strcmp(key, "ILoveCandy") == 0 || strcmp(upperkey, "ILOVECANDY") == 0) {
+				} else if(strcmp(key, "ILoveCandy") == 0) {
 					config->chomp = 1;
 					pm_printf(PM_LOG_DEBUG, "config: chomp\n");
-				} else if(strcmp(key, "UseColor") == 0 || strcmp(upperkey, "USECOLOR") == 0) {
-					config->usecolor = 1;
-					pm_printf(PM_LOG_DEBUG, "config: usecolor\n");
-				} else if(strcmp(key, "ShowSize") == 0 || strcmp(upperkey, "SHOWSIZE") == 0) {
+				} else if(strcmp(key, "ShowSize") == 0) {
 					config->showsize = 1;
 					pm_printf(PM_LOG_DEBUG, "config: showsize\n");
-				} else if(strcmp(key, "UseDelta") == 0 || strcmp(upperkey, "USEDELTA") == 0) {
+				} else if(strcmp(key, "UseDelta") == 0) {
 					alpm_option_set_usedelta(1);
 					pm_printf(PM_LOG_DEBUG, "config: usedelta\n");
-				} else if(strcmp(key, "TotalDownload") == 0 || strcmp(upperkey, "TOTALDOWNLOAD") == 0) {
+				} else if(strcmp(key, "TotalDownload") == 0) {
 					config->totaldownload = 1;
 					pm_printf(PM_LOG_DEBUG, "config: totaldownload\n");
 				} else {
@@ -652,59 +659,66 @@ static int _parseconfig(const char *file, const char *givensection,
 				}
 			} else {
 				/* directives with settings */
-				if(strcmp(key, "Include") == 0 || strcmp(upperkey, "INCLUDE") == 0) {
+				if(strcmp(key, "Include") == 0) {
 					pm_printf(PM_LOG_DEBUG, "config: including %s\n", ptr);
 					_parseconfig(ptr, section, db);
 					/* Ignore include failures... assume non-critical */
 				} else if(strcmp(section, "options") == 0) {
-					if(strcmp(key, "NoUpgrade") == 0
-							|| strcmp(upperkey, "NOUPGRADE") == 0) {
+					if(strcmp(key, "NoUpgrade") == 0) {
 						setrepeatingoption(ptr, "NoUpgrade", alpm_option_add_noupgrade);
-					} else if(strcmp(key, "NoExtract") == 0
-							|| strcmp(upperkey, "NOEXTRACT") == 0) {
+					} else if(strcmp(key, "NoExtract") == 0) {
 						setrepeatingoption(ptr, "NoExtract", alpm_option_add_noextract);
-					} else if(strcmp(key, "IgnorePkg") == 0
-							|| strcmp(upperkey, "IGNOREPKG") == 0) {
+					} else if(strcmp(key, "IgnorePkg") == 0) {
 						setrepeatingoption(ptr, "IgnorePkg", alpm_option_add_ignorepkg);
-					} else if(strcmp(key, "IgnoreGroup") == 0
-							|| strcmp(upperkey, "IGNOREGROUP") == 0) {
+					} else if(strcmp(key, "IgnoreGroup") == 0) {
 						setrepeatingoption(ptr, "IgnoreGroup", alpm_option_add_ignoregrp);
-					} else if(strcmp(key, "HoldPkg") == 0
-							|| strcmp(upperkey, "HOLDPKG") == 0) {
+					} else if(strcmp(key, "HoldPkg") == 0) {
 						setrepeatingoption(ptr, "HoldPkg", alpm_option_add_holdpkg);
-					} else if(strcmp(key, "DBPath") == 0 || strcmp(upperkey, "DBPATH") == 0) {
+					} else if(strcmp(key, "SyncFirst") == 0) {
+						setrepeatingoption(ptr, "SyncFirst", option_add_syncfirst);
+					} else if(strcmp(key, "DBPath") == 0) {
 						/* don't overwrite a path specified on the command line */
 						if(!config->dbpath) {
 							config->dbpath = strdup(ptr);
 							pm_printf(PM_LOG_DEBUG, "config: dbpath: %s\n", ptr);
 						}
-					} else if(strcmp(key, "CacheDir") == 0 || strcmp(upperkey, "CACHEDIR") == 0) {
+					} else if(strcmp(key, "CacheDir") == 0) {
 						if(alpm_option_add_cachedir(ptr) != 0) {
 							pm_printf(PM_LOG_ERROR, _("problem adding cachedir '%s' (%s)\n"),
 									ptr, alpm_strerrorlast());
 							return(1);
 						}
 						pm_printf(PM_LOG_DEBUG, "config: cachedir: %s\n", ptr);
-					} else if(strcmp(key, "RootDir") == 0 || strcmp(upperkey, "ROOTDIR") == 0) {
+					} else if(strcmp(key, "RootDir") == 0) {
 						/* don't overwrite a path specified on the command line */
 						if(!config->rootdir) {
 							config->rootdir = strdup(ptr);
 							pm_printf(PM_LOG_DEBUG, "config: rootdir: %s\n", ptr);
 						}
-					} else if (strcmp(key, "LogFile") == 0 || strcmp(upperkey, "LOGFILE") == 0) {
+					} else if (strcmp(key, "LogFile") == 0) {
 						if(!config->logfile) {
 							config->logfile = strdup(ptr);
 							pm_printf(PM_LOG_DEBUG, "config: logfile: %s\n", ptr);
 						}
-					} else if (strcmp(key, "XferCommand") == 0 || strcmp(upperkey, "XFERCOMMAND") == 0) {
+					} else if (strcmp(key, "XferCommand") == 0) {
 						alpm_option_set_xfercommand(ptr);
 						pm_printf(PM_LOG_DEBUG, "config: xfercommand: %s\n", ptr);
+					} else if (strcmp(key, "CleanMethod") == 0) {
+						if (strcmp(ptr, "KeepInstalled") == 0) {
+							config->cleanmethod = PM_CLEAN_KEEPINST;
+						} else if (strcmp(ptr, "KeepCurrent") == 0) {
+							config->cleanmethod = PM_CLEAN_KEEPCUR;
+						} else {
+							pm_printf(PM_LOG_ERROR, _("invalid value for 'CleanMethod' : '%s'\n"), ptr);
+							return(1);
+						}
+						pm_printf(PM_LOG_DEBUG, "config: cleanmethod: %s\n", ptr);
 					} else {
 						pm_printf(PM_LOG_ERROR, _("config file %s, line %d: directive '%s' not recognized.\n"),
 								file, linenum, key);
 						return(1);
 					}
-				} else if(strcmp(key, "Server") == 0 || strcmp(upperkey, "SERVER") == 0) {
+				} else if(strcmp(key, "Server") == 0) {
 					/* let's attempt a replacement for the current repo */
 					char *server = strreplace(ptr, "$repo", section);
 
@@ -720,7 +734,6 @@ static int _parseconfig(const char *file, const char *givensection,
 					return(1);
 				}
 			}
-			free(upperkey);
 		}
 	}
 	fclose(fp);
@@ -753,7 +766,7 @@ int main(int argc, char *argv[])
 {
 	int ret = 0;
 	struct sigaction new_action, old_action;
-#if defined(HAVE_GETEUID)
+#if defined(HAVE_GETEUID) && !defined(CYGWIN)
 	/* geteuid undefined in CYGWIN */
 	uid_t myuid = geteuid();
 #endif
@@ -834,7 +847,12 @@ int main(int argc, char *argv[])
 		cleanup(ret);
 	}
 
-#if defined(HAVE_GETEUID)
+	/* set TotalDownload callback if option enabled */
+	if(config->totaldownload) {
+		alpm_option_set_totaldlcb(cb_dl_total);
+	}
+
+#if defined(HAVE_GETEUID) && !defined(CYGWIN)
 	/* check if we have sufficient permission for the requested operation */
 	if(myuid > 0 && needs_transaction()) {
 		pm_printf(PM_LOG_ERROR, _("you cannot perform this operation unless you are root.\n"));
@@ -867,9 +885,6 @@ int main(int argc, char *argv[])
 
 	/* start the requested operation */
 	switch(config->op) {
-		case PM_OP_ADD:
-			ret = pacman_add(pm_targets);
-			break;
 		case PM_OP_REMOVE:
 			ret = pacman_remove(pm_targets);
 			break;
