@@ -21,15 +21,14 @@
 
 #include <alpm_list.h>
 
-/* add.c, this should merge with upgrade.c */
-int pacman_add(alpm_list_t *targets);
-int pacman_upgrade(alpm_list_t *targets);
-/* sync.c */
-int pacman_sync(alpm_list_t *targets);
 /* query.c */
 int pacman_query(alpm_list_t *targets);
 /* remove.c */
 int pacman_remove(alpm_list_t *targets);
+/* sync.c */
+int pacman_sync(alpm_list_t *targets);
+/* upgrade.c */
+int pacman_upgrade(alpm_list_t *targets);
 
 /* deptest.c */
 int pacman_deptest(alpm_list_t *targets);
