@@ -36,6 +36,8 @@
 /* update speed for the fill_progress based functions */
 #define UPDATE_SPEED_SEC 0.2f
 
+int trans_init(pmtranstype_t type, pmtransflag_t flags);
+int trans_release();
 int needs_transaction();
 int getcols();
 int makepath(const char *path);
@@ -49,8 +51,9 @@ char *strreplace(const char *str, const char *needle, const char *replace);
 alpm_list_t *strsplit(const char *str, const char splitchar);
 void string_display(const char *title, const char *string);
 void list_display(const char *title, const alpm_list_t *list);
-void display_targets(const alpm_list_t *syncpkgs, pmdb_t *db_local);
-int yesno(char *fmt, ...);
+void display_targets(const alpm_list_t *pkgs, int install);
+void display_synctargets(const alpm_list_t *syncpkgs);
+int yesno(short preset, char *fmt, ...);
 int pm_printf(pmloglevel_t level, const char *format, ...) __attribute__((format(printf,2,3)));
 int pm_fprintf(FILE *stream, pmloglevel_t level, const char *format, ...) __attribute__((format(printf,3,4)));
 int pm_vfprintf(FILE *stream, pmloglevel_t level, const char *format, va_list args) __attribute__((format(printf,3,0)));
